import html2pdf from "html2pdf.js";
import html2canvas from "html2canvas";
import jsPDF from "jspdf";

// jsPDF config doc
// https://rawgit.com/MrRio/jsPDF/master/docs/jsPDF.html
export const jsPDFConfigs = {
  A3: { unit: "mm", format: "a3", orientation: "landscape" },
  A4: { unit: "mm", format: "a4", orientation: "portrait" },
  "8K": {
    unit: "mm",
    format: [370, 255],
    orientation: "landscape",
  },
};

export async function buildPdfSimple(
  { element, pageSize, filename },
  returnBlob = false
) {
  const opt = {
    margin: 0,
    html2canvas: { scale: 6, useCORS: true },
    filename: filename || "",
    image: { type: "jpeg", quality: 0.8 },
    jsPDF: {
      ...jsPDFConfigs[pageSize],
    },
  };

  if (returnBlob) {
    return await html2pdf().set(opt).from(element).outputPdf("blob");
  } else {
    await html2pdf().set(opt).from(element).save();
  }
}

export async function buildPdf(
  { elements, pageSize, filename, scale },
  returnBlob = false
) {
  const opt = {
    margin: 0,
    html2canvas: { scale: scale || 6, useCORS: true },
    filename: filename || "",
    image: { type: "jpeg", quality: 0.8 },
    jsPDF: {
      ...jsPDFConfigs[pageSize],
    },
  };

  const mergedPdf = await buildPdfFromElements(elements, opt);

  if (returnBlob) {
    return await mergedPdf.output("blob");
  } else {
    // console.log(filename);
    await mergedPdf.save(filename);
  }
}

export async function buildPdfFromElements(domList, pdfOptions) {
  const mergedPdf = new jsPDF(pdfOptions.jsPDF);

  for (let i = 0; i < domList.length; i++) {
    const element = domList[i];
    const canvas = await html2canvas(element, pdfOptions.html2canvas);
    const blob = await new Promise((resolve) =>
      canvas.toBlob(
        resolve,
        `image/${pdfOptions.image.type}`,
        pdfOptions.image.quality
      )
    );
    const img = document.createElement("img");
    img.src = URL.createObjectURL(blob);

    const pdfWidth = mergedPdf.internal.pageSize.getWidth();
    const pdfHeight = mergedPdf.internal.pageSize.getHeight();

    if (i > 0) mergedPdf.addPage();
    mergedPdf.addImage(img, "JPEG", 0, 0, pdfWidth, pdfHeight);
  }

  return mergedPdf;
}
